/*
  Copyright 1995 The University of Rhode Island and The Massachusetts
  Institute of Technology

  Portions of this software were developed by the Graduate School of
  Oceanography (GSO) at the University of Rhode Island (URI) in collaboration
  with The Massachusetts Institute of Technology (MIT).

  Access and use of this software shall impose the following obligations and
  understandings on the user. The user is granted the right, without any fee
  or cost, to use, copy, modify, alter, enhance and distribute this software,
  and any derivative works thereof, and its supporting documentation for any
  purpose whatsoever, provided that this entire notice appears in all copies
  of the software, derivative works and supporting documentation.  Further,
  the user agrees to credit URI/MIT in any publications that result from the
  use of this software or in any product that includes this software. The
  names URI, MIT and/or GSO, however, may not be used in any advertising or
  publicity to endorse or promote any products or commercial entity unless
  specific written permission is obtained from URI/MIT. The user also
  understands that URI/MIT is not obligated to provide the user with any
  support, consulting, training or assistance of any kind with regard to the
  use, operation and performance of this software nor to provide the user
  with any updates, revisions, new versions or "bug fixes".

  THIS SOFTWARE IS PROVIDED BY URI/MIT "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
  EVENT SHALL URI/MIT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
  DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTUOUS
  ACTION, ARISING OUT OF OR IN CONNECTION WITH THE ACCESS, USE OR PERFORMANCE
  OF THIS SOFTWARE.
*/


// DODS-netCDF surrogate library functions for data attributes. See note
// about the symbol LOCAL in Dfile.c
// 
// ReZa 10/4/94

// $Log: Dattr.cc,v $
// Revision 1.15  1999/05/07 23:45:30  jimg
// String --> string fixes
//
// Revision 1.14  1999/03/30 05:20:55  reza
// Added support for the new data types (Int16, UInt16, and Float32).
//
// Revision 1.13  1997/03/26 05:44:20  reza
// GLOBAL attribute can now be named anything with "_GLOBAL" in it (i.e.
// HDF_GLOBAL, MAT_GLOBAL, DSP_GLOBAL, etc.). This should improve interoperability
// with non-netCDF servers. Also, the DAS no longer requires attributes for all
// the variables. An internal empty attribute table (NC_BLANK) manages missing ones.
//
// Revision 1.12  1997/03/25 16:05:56  reza
// Fixed CVS duplicate log reports.
//
// Revision 1.11  1996/09/17 00:26:04  jimg
// Merged changes from a side branch which contained various changes from
// Reza and Charles.
// Removed ncdump and netexec since ncdump is now in its own directory and
// netexec is no longer used.
//
// Revision 1.10.2.1  1996/06/25 22:04:04  jimg
// Version 2.0 from Reza.
//
// Revision 1.9.2.2  1996/06/10 19:36:06  jimg
// Replaced strcpy in correct_att_type() with memcpy.
//
// Revision 1.9.2.1  1996/04/02 19:29:53  jimg
// Changed the `conns' variable so that now it is a global *pointer*, not an
// object. Each call that might add to the conns object must first check to see
// that it has been created. This was done because of link-time problems with
// static global objects.
// Fixed up some comment stuff.
// Added assert() calls for conns.
//
// Revision 1.9  1995/07/09  21:33:29  jimg
// Added copyright notice.
//
// Revision 1.8  1995/06/29  15:42:02  jimg
// Fixed instances of delete which should have included brackets
//
// Revision 1.7  1995/06/28  20:22:33  jimg
// Replaced malloc calls with calls to new (and calls to free with calls to
// delete).
//
// Revision 1.6  1995/06/23  13:59:32  reza
// Minor changes to make Efence work.
//
// Revision 1.5  1995/03/16  16:38:36  reza
// Updated byte transfer. Bytes are no longer transmitted in binary but in
// an ASCII string (0-255).
//
// Revision 1.4  1994/12/22  04:46:18  reza
// Updated to use DODS new attribute array capability.
//
// Revision 1.3  1994/12/08  19:01:27  reza
// Added support for local netCDF file access through the original netCDF code.
//
// Revision 1.2  1994/11/23  21:05:50  reza
// First working version.
//
// Revision 1.1  1994/11/03  04:35:44  reza
// Preliminary version of netCDF -> DODS library.

#include "config_nc.h"

static char rcsid[] not_used ={"$Id: Dattr.cc,v 1.15 1999/05/07 23:45:30 jimg Exp $"};

#include <assert.h>
#include  "Dnetcdf.h"

#ifdef __GNUG__
#pragma implementation
#endif

Pix
get_das_table(int cdfid, int varid)
{
  Pix q;

  assert(conns); 

  q = (*conns)[cdfid]->das().first_var();
  for (int i=0; i < (*conns)[cdfid]->das_loc(varid+1) ; i++)
    (*conns)[cdfid]->das().next_var(q);
 
  return q;
}

void
correct_att_type(int cdfid, Pix q, const char *name, void *values)
{
    unsigned int i;
    nclong *lp;
    short *sp;
    float *fp;
    double *dp;
    char *cp;

    assert(conns); 

    // from the attribute table get the requested atrribute type by name

    string attrT = (*conns)[cdfid]->das().get_table(q)->get_type(name);

    // from the attribute table get the requested atrribute length by name
    unsigned int attr_siz = (*conns)[cdfid]->das().get_table(q)->get_attr_num(name);

    // integers are converted to the binary and added to the pointer
    if (attrT == "Int16") {
	sp = (short *) values;
	for (i=0; i<attr_siz; i++)
	    *(sp+i) = atol((*conns)[cdfid]->das().get_table(q)->get_attr(name,i).c_str());
    }
    if (attrT == "Int32") {
	lp = (nclong *) values;
	for (i=0; i<attr_siz; i++)
	    *(lp+i) = atol((*conns)[cdfid]->das().get_table(q)->get_attr(name,i).c_str());
    }

    // floats are converted to the binary and added to the pointer address
    if (attrT == "Float32"){
	fp = (float *) values;
	for (i=0; i<attr_siz; i++)
	    *(fp+i)=atof((*conns)[cdfid]->das().get_table(q)->get_attr(name,i).c_str());
    }
    if (attrT == "Float64"){
	dp = (double *) values;
	for (i=0; i<attr_siz; i++)
	    *(dp+i)=atof((*conns)[cdfid]->das().get_table(q)->get_attr(name,i).c_str());
    }

    // bytes are added one at the time to the pointer address
    if (attrT == "Byte"){
	cp = (char *) values;
	for (i=0; i<attr_siz; i++)
	    *(cp+i)=(char) atoi((*conns)[cdfid]->das().get_table(q)->get_attr(name,i).c_str());
    }

    // strings are added to the pointer (seperated by '\n' and ended by '\0')
    if (attrT == "String"){
	cp = (char *) values;
	int offset=0;
	for (i=0; i<attr_siz; i++){
	    string attrV = (*conns)[cdfid]->das().get_table(q)->get_attr(name,i);	    
	    size_t atsiz = attrV.length() - 2 ; // size minus quotes  
	    string Attribute = attrV.substr(1, atsiz); // remove quotes
	    (void) memcpy(cp+offset, Attribute.c_str(), atsiz); 
	    offset += atsiz; 
	    if(attr_siz > 1){
	      *(cp+offset) = '\n'; // Separate strings by EOL for more than 1
	      offset++;            // string (not a DODS netcdf server).
	    }
	}
	if(attr_siz > 1)
	*(cp+offset-1) = '\0';	// End the string attribute by EOS, if array
				// of string was sent (not a netCDF server).
    }

}

int
ncattput(int cdfid, int varid, const char *name,  nc_type datatype, 
	 int count, const ncvoid *values)
{
    assert(conns); 

    if((*conns)[cdfid] == NULL) // does the file id exist?
	return(-1) ;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncattput((*conns)[cdfid]->ncid(), varid, name, 
			  datatype, count, values);
	return rcode;
    }
#endif

    cdf_routine_name = "ncattput" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return(-1);
}


int
ncattrename(int cdfid, int varid, const char *name, const char *newname)
{
    assert(conns); 

    if((*conns)[cdfid] == NULL) // does the file id exist?
	return(-1) ;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncattrename((*conns)[cdfid]->ncid(), varid, name, newname);
	return rcode;
    }
#endif

    cdf_routine_name = "ncattrename" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return(-1);
}


int
ncattdel(int cdfid, int varid, const char *name)
{
    assert(conns); 

    if((*conns)[cdfid] == NULL) // does the file id exist?
	return(-1) ;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncattdel((*conns)[cdfid]->ncid(), varid, name);
	return rcode;
    }
#endif

    cdf_routine_name = "ncattdel" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return(-1);
}


int
ncattname( int cdfid, int varid, int attnum, char *name)
{
    assert(conns); 

    if((*conns)[cdfid] == NULL) // does the file id exist?
	return(-1) ;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncattname( (*conns)[cdfid]->ncid(), varid, attnum, name);
	return rcode;
    }
#endif

    cdf_routine_name = "ncattname" ;

    // varid = -1 is used for global atrributes
    if(( varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars())) {
	    NCadvise(NC_ENOTVAR, "%d is not a valid variable id", varid) ;
	    return(-1) ;
    }
 
    Pix q = get_das_table(cdfid,varid); // get das table

    // from the attribute table get the requested atrribute name
    Pix p = (*conns)[cdfid]->das().get_table(q)->first_attr();
    for (int i=0; (i < attnum) && p ; i++)
	(*conns)[cdfid]->das().get_table(q)->next_attr(p);

    if(p == NULL) // attnum is greater than the attributes in the table
	{
	    NCadvise(NC_ENOTATT, "%d is not a valid attribute id", attnum) ;
	    return(-1) ;
	}
    else
	(void) strcpy (name, (*conns)[cdfid]->das().get_table(q)->get_name(p).c_str());

    return(attnum) ;
}


int
ncattinq( int cdfid, int varid, const char *name, 
	  nc_type *datatypep, int *countp)  
{
    assert(conns); 

    if((*conns)[cdfid] == NULL) // does the file id exist?
	return(-1) ;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncattinq( (*conns)[cdfid]->ncid(), varid, 
			   name, datatypep, countp);
	return rcode;
    }
#endif

    cdf_routine_name = "ncattinq" ;

    // varid = -1 is used for global atrributes
    if(( varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars()))
	{
	    NCadvise(NC_ENOTVAR, "%d is not a valid variable id", varid) ;
	    return(-1) ;
	}

    Pix q = get_das_table(cdfid, varid); // get das table

    // from the attribute table get the requested atrribute by name
    string attrV = (*conns)[cdfid]->das().get_table(q)->get_attr(name);

    // from the attribute table get the requested atrribute type by name
    string attrT = (*conns)[cdfid]->das().get_table(q)->get_type(name);

    // from the attribute table get the requested atrribute length by name
    unsigned int attr_siz = (*conns)[cdfid]->das().get_table(q)->get_attr_num(name);

    if(attrV == "") {
	NCadvise(NC_ENOTATT, "%s attribute does not exist", name) ;
	return(-1) ;
    }
    else { 
	if(datatypep != 0) {
	    if (attrT == "String")
		*datatypep =  NC_CHAR;
	  
	    if (attrT == "Byte")
		*datatypep =  NC_BYTE;
	  
	    if (attrT == "Int32")
		*datatypep =  NC_LONG;

	    if (attrT == "Int16")
		*datatypep =  NC_SHORT;

	    if (attrT == "Float32")
		*datatypep =  NC_FLOAT;
	  
	    if (attrT == "Float64")
		*datatypep =  NC_DOUBLE;
	}

	if(countp != 0) {
	    *countp = 0;
	    if(attrT == "String"){ // string lenght is the total number of
				   // bytes plus terminating character. There
				   // is a '\n' at the end of each string, but
				   // a '\0' at the end of the attibutes.
		for (int i =0; i < attr_siz; i++){
		    attrV = (*conns)[cdfid]->das().get_table(q)->get_attr(name,i);
                    *countp +=  attrV.length() - 1; // EOS included, quotes removed 
		}
		if((*conns)[cdfid]->das().get_table(q)->get_attr_num(name) == 1)
		  *countp -= 1;   // single string (coming from a netCDF server)
		                  // remove EOS for compatibility
	    }
	    else {
		*countp =  attr_siz;
	    }
	}
	
    }

    return(1) ;
}


int
ncattcopy(int incdf, int invar, const char *name, int outcdf, int outvar)
{
    void *values;         // Attribute vector to be copied,
    nc_type datatype;    // it's type, 
    int count;          // and it's length.

    assert(conns); 

    //does the file ids exist?
    if(((*conns)[incdf] == NULL) || ((*conns)[outcdf] == NULL))
	return(-1) ;

    // case 1, both input and output files are local

#ifdef LOCAL
    if((*conns)[incdf]->is_local() && (*conns)[outcdf]->is_local()) {
	rcode = lncattcopy((*conns)[incdf]->ncid(), invar, 
			   name, (*conns)[outcdf]->ncid(), outvar);
	return rcode;
    }
#endif

    cdf_routine_name = "ncattcopy" ;

    // case 2, the output file is not local, Error writing over the net.

    if(!(*conns)[outcdf]->is_local()) {
	NCadvise(NC_EPERM, "DODS network interface is read-only") ;
	return(-1);
    }

    // last case, input is from DODS file and output a local netcdf file

#ifdef LOCAL

    // invar = -1 is used for global atrributes
    if(( invar+1 < 0) || (invar+1 > (*conns)[incdf]->nvars())) {
	    NCadvise(NC_ENOTVAR, "%d is not a valid variable id", invar) ;
	    return(-1) ;
    }

    Pix q = get_das_table(incdf,invar); // get das table

    // from the attribute table get the requested atrribute by name
    string attrV = (*conns)[incdf]->das().get_table(q)->get_attr(name);

    if(attrV == "") {
	    NCadvise(NC_ENOTATT, "%s attribute does not exist", name) ;
	    return(-1) ;
    }

    // get the attribute in the right format
    ncattinq(incdf, invar, name, &datatype, &count);

#ifdef NEVER
    values = (void *) malloc((unsigned)count * nctypelen(datatype));
#endif
    values = (void *) new char [((unsigned)count + 1) * nctypelen(datatype)];
    if (!values) 
	return (-1);

    correct_att_type(incdf, q, name, values);

    // now write it to the local file
#ifdef NEVER 
    lncredef((*conns)[outcdf]->ncid()); // This is users job
#endif

    rcode = lncattput((*conns)[outcdf]->ncid(), outvar,  
		      name, datatype, count, values);

#ifdef NEVER
    lncendef((*conns)[outcdf]->ncid()); //This is users job
#endif

    delete [] values;
    return rcode;

#else

    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return(-1);

#endif /* Last case LOCAL */
}

int
ncattget(int cdfid, int varid, const char *name, ncvoid *values)
{
    assert(conns); 

    if ((*conns)[cdfid] == NULL) // does the file id exist?
	return(-1) ;

#ifdef LOCAL
    if ((*conns)[cdfid]->is_local()) {
	rcode = lncattget((*conns)[cdfid]->ncid(), varid, name, values);
	return rcode;
    }
#endif

    cdf_routine_name = "ncattget" ;

    // varid = -1 is used for global atrributes
    if ((varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars())) {
	NCadvise(NC_ENOTVAR, "%d is not a valid variable id", varid) ;
	return(-1) ;
    }

    Pix q = get_das_table(cdfid,varid); // get das table

    // from the attribute table get the requested atrribute by name
    string attrV = (*conns)[cdfid]->das().get_table(q)->get_attr(name);

    if(attrV == "") {
	NCadvise(NC_ENOTATT, "%s attribute does not exist", name) ;
	return(-1) ;
    }
    else
	correct_att_type(cdfid, q, name, values);

    return(1) ;
}
