
#include "../ncview.includes.h"
#include "../ncview.defines.h"

/* You can 'set' the data to a specified value using the dataedit window --
 * this tells whether or not that mode is currently engaged.
 */
#define	DE_SET_MODE_OFF	0
#define	DE_SET_MODE_ON	1

static Widget
	dataedit_popup_widget = NULL,
		dataedit_popupcanvas_widget,
			dataedit_viewport_widget,
				dataedit_list_widget,
			dataedit_dump_widget,
			dataedit_done_widget,
			dataedit_set_widget,
			dataedit_set_value_widget;

static int	dataedit_popup_done = FALSE, dataedit_popup_result = 0;
static int	dataedit_set_mode   = DE_SET_MODE_OFF;
static XEvent	de_event;

void 	dataedit_dump_callback(Widget w, XtPointer client_data, XtPointer call_data );
void 	dataedit_done_callback(Widget w, XtPointer client_data, XtPointer call_data );
void 	dataedit_callback(Widget w, XtPointer client_data, XtPointer call_data );
void 	dataedit_set_callback(Widget w, XtPointer client_data, XtPointer call_data );

extern Widget 		topLevel;
extern XtAppContext 	x_app_context;

/*****************************************************************/

	void
x_dataedit( char **text, int nx )
{
	dataedit_popup_widget = XtVaCreatePopupShell(
		"Data Edit Popup",
		transientShellWidgetClass,
		topLevel,
		NULL );

	dataedit_popupcanvas_widget = XtVaCreateManagedWidget(
		"dataedit_popupcanvas",
		boxWidgetClass,
		dataedit_popup_widget,
		NULL);

	dataedit_viewport_widget = XtVaCreateManagedWidget(
		"Data Viewport",
		viewportWidgetClass,
		dataedit_popupcanvas_widget,
		XtNwidth, 400,
		XtNheight, 300,
		XtNallowHoriz, True,
		XtNallowVert,  True,
		NULL );

	dataedit_list_widget = XtVaCreateManagedWidget(
		"Data Edit",
		listWidgetClass,
		dataedit_viewport_widget,
		XtNlist, text,
		XtNdefaultColumns, nx,
		XtNforceColumns, True,
		NULL );

	dataedit_dump_widget = XtVaCreateManagedWidget(
		"Dump Data",
		commandWidgetClass,
		dataedit_popupcanvas_widget,
		XtNfromVert, dataedit_list_widget,
		NULL);

	dataedit_done_widget = XtVaCreateManagedWidget(
		"Done",
		commandWidgetClass,
		dataedit_popupcanvas_widget,
		XtNfromVert, dataedit_list_widget,
		XtNfromHoriz, dataedit_dump_widget,
		NULL);

	dataedit_set_widget = XtVaCreateManagedWidget(
		"Set",
		toggleWidgetClass,
		dataedit_popupcanvas_widget,
		XtNfromVert, dataedit_list_widget,
		XtNfromHoriz, dataedit_done_widget,
		NULL);

	dataedit_set_value_widget = XtVaCreateManagedWidget(
		"dataedit_set_value",
		dialogWidgetClass,
		dataedit_popupcanvas_widget,
		XtNfromVert, dataedit_list_widget,
		XtNfromHoriz, dataedit_set_widget,
		XtNvalue, "0.0",
		XtNlabel, "Value to set to:",
		NULL);

        XtAddCallback( dataedit_dump_widget, XtNcallback, 
			dataedit_dump_callback, (XtPointer)MESSAGE_OK );
        XtAddCallback( dataedit_done_widget, XtNcallback, 
			dataedit_done_callback, (XtPointer)MESSAGE_OK );
        XtAddCallback( dataedit_set_widget, XtNcallback, 
			dataedit_set_callback, (XtPointer)MESSAGE_OK );
        XtAddCallback( dataedit_list_widget, XtNcallback, 
			dataedit_callback, (XtPointer)text );

	XtPopup      ( dataedit_popup_widget, XtGrabNone );

	/* This mini main loop just handles the dataedit popup widget */
	dataedit_popup_done = FALSE;
	while( ! dataedit_popup_done ) {
		XtAppNextEvent( x_app_context, &de_event );
		XtDispatchEvent( &de_event );
		}

	XtPopdown( dataedit_popup_widget );

	XtDestroyWidget( dataedit_dump_widget        );
	XtDestroyWidget( dataedit_done_widget        );
	XtDestroyWidget( dataedit_list_widget        );
	XtDestroyWidget( dataedit_popupcanvas_widget );
	XtDestroyWidget( dataedit_popup_widget       );
	dataedit_popup_widget = NULL;
}
	
	void
x_set_edit_place( long index, int x, int y, int nx, int ny )
{
	Widget	horiz_scroll, vert_scroll;
	float	shown_horiz, shown_vert;
	float	new_horiz,   new_vert;
	XtArgVal *new_value;

	if( dataedit_popup_widget == NULL )
		return;

	XawListHighlight( dataedit_list_widget, index );
	horiz_scroll = XtNameToWidget( dataedit_viewport_widget, "horizontal" );
	vert_scroll  = XtNameToWidget( dataedit_viewport_widget, "vertical"   );
	XtVaGetValues( horiz_scroll, XtNshown,  &shown_horiz, NULL );
	XtVaGetValues( vert_scroll, XtNshown,  &shown_vert, NULL );
	new_horiz = (float)x/(float)nx - shown_horiz/2.0;
	new_vert  = (float)y/(float)ny - shown_vert/2.0;
	new_horiz = (new_horiz > 1.0) ? 1.0 : new_horiz;
	new_vert  = (new_vert  > 1.0) ? 1.0 : new_vert;
	new_horiz = (new_horiz < 0.0) ? 0.0 : new_horiz;
	new_vert  = (new_vert  < 0.0) ? 0.0 : new_vert;
	XawViewportSetLocation( dataedit_viewport_widget, new_horiz, new_vert );
}

        void
dataedit_callback(widget, client_data, call_data)
Widget widget;
XtPointer client_data;
XtPointer call_data;
{
	XawListReturnStruct	*list;
	int			message, n_fields;
	char			line[132];
	float			new_val, dummy;
	char			**list_text;

	list_text = (char **)client_data;
	list      = XawListShowCurrent( dataedit_list_widget );

	if( dataedit_set_mode == DE_SET_MODE_ON ) {
		strcpy( line, 
			XawDialogGetValueString( dataedit_set_value_widget) );
		}
	else
		{
		strcpy( line, list->string );
		message = x_dialog( "", line, TRUE );
		if( message != MESSAGE_OK )
			return;
		}
	n_fields = sscanf( line, "%f %f", &new_val, &dummy );
	if( n_fields != 1 ) 
		return;

	in_change_dat( (long)list->list_index, new_val );
	strcpy( *(list_text+list->list_index), line );
	XawListChange( dataedit_list_widget, list_text, 0, 0, True );
	XawListHighlight( dataedit_list_widget, list->list_index );
}

        void
dataedit_dump_callback(widget, client_data, call_data)
Widget widget;
XtPointer client_data;
XtPointer call_data;
{
	in_data_edit_dump();
}

        void
dataedit_done_callback(widget, client_data, call_data)
Widget widget;
XtPointer client_data;
XtPointer call_data;
{
	dataedit_popup_done   = TRUE;
}

        void
dataedit_set_callback(widget, client_data, call_data)
Widget widget;
XtPointer client_data;
XtPointer call_data;
{
	if(dataedit_set_mode == DE_SET_MODE_OFF )
		dataedit_set_mode = DE_SET_MODE_ON;
	else
		dataedit_set_mode = DE_SET_MODE_OFF;
}
